==> leadlink_open_api-0.0.7.tar.gz: lib/api_leadlink_qrcode.dart <==
part of leadlinkopenapi;

class APILeadLinkQRCode {

  // 二维码生成字符串
  static void createQRCode(int productId,int skuId,List<int> userIdList,String expirationDate,bool isForever,{Function(String qrcodeData) onSucc,Function(String) onFail}) {

    Map<String,dynamic> input = Map<String,dynamic>();
    input['sk'] = skuId ?? 0;
    input['if'] = isForever == true ? 1 : 0;
    input['ex'] = expirationDate ?? '';
    input['pi'] = productId ?? 0;
    input['us'] = userIdList ?? [0];

    try {
      // 转字符串
      String jsonString = json.encode(input);
      // 加密
      String encryptWord = APIUtil.encrypted(jsonString);
      // 加壳
      String encryptWordShell = APIUtil.shellQRcodeString(encryptWord);
      if(onSucc != null) onSucc(encryptWordShell);
    } catch (e) {
      if(onFail != null) onFail('二维码未生成成功');
    }

  }

  // 验证是是否是力联二维码
  static bool isQRcodeAvailable(String qrcodeData) {
    return APIUtil.isAvailableQRcode(qrcodeData);
  }

  // 二维码解析
  static parseQRCode(int userId,String qrcodeData,{Function(LKQRCodeResult result,String msg) onSucc,
  Function(LKQRCodeResult result,String msg) onFail}) async {

    SharedPreferences prefs = await SharedPreferences.getInstance();
    int userId = prefs.getInt(APITip.spKeyUserId) ?? 0 ;

    // 未登录
    if(userId == 0 ) {
      if(onFail != null) onFail(
        LKQRCodeResult(isCanOpen: false,isForEver: false,isAuthorized: false,isRecognized: false,isUnExpired: false,
        productId: 0,skuId: 0,expirationDate: '',productName: '',
        qrcodeData: qrcodeData,isProductExist: false,isProductSkuExist:false,authorizedState: '',
      ),APITip.login404);
      return;
    }

    // 二维码合法性检测
    if(APIUtil.isAvailableQRcode(qrcodeData) == false) {
      if(onFail != null) onFail(
        LKQRCodeResult(isCanOpen: false,isForEver: false,isAuthorized: false,isRecognized: false,isUnExpired: false,
        productId: 0,skuId: 0,expirationDate: '',productName: '',
        qrcodeData: qrcodeData,isProductExist: false,isProductSkuExist:false,authorizedState: '',
      ),APITip.login404);
      return;
    }


    // 脱壳
    String unShellWord = APIUtil.unshellQRcodeString(qrcodeData);
    if(unShellWord == '') {
      if(onFail != null) onFail(
        LKQRCodeResult(isCanOpen: false,isForEver: false,isAuthorized: false,isRecognized: false,isUnExpired: false,
        productId: 0,skuId: 0,expirationDate: '',productName: '',
        qrcodeData: qrcodeData,isProductExist: false,isProductSkuExist:false,authorizedState: '',
      ),APITip.apiqrNoCheck);
      return;
    }
    // 解密
    String plain = APIUtil.decrypted(unShellWord);
    if(plain == '') {
      if(onFail != null) onFail(
        LKQRCodeResult(isCanOpen: false,isForEver: false,isAuthorized: false,isRecognized: false,isUnExpired: false,
        productId: 0,skuId: 0,expirationDate: '',productName: '',
        qrcodeData: qrcodeData,isProductExist: false,isProductSkuExist:false,authorizedState: '',
      ),APITip.apiqrNoCheck);
      return;
    }

    // 解析
    try {


      Map<String,dynamic> map       = json.decode(plain);
      List<int> userIdList          = map['us'].cast<int>() ?? [];
      int isForever                 = map['if'] ?? 0 ;
      int skuId                     = map['sk'] ?? 0;
      int productId                 = map['pi'] ?? 0 ;
      String expirationDate         = map['ex'] ?? '' ;

      LKQRCodeResult result = LKQRCodeResult();
      result.qrcodeData = qrcodeData;
      result.skuId      = skuId;
      result.isRecognized = true;
      result.expirationDate = expirationDate;
      result.productId = productId;
      result.isForEver = isForever == 0 ? false : true;

      // 在有效期内判断
      if(result.isForEver == true){
        result.isUnExpired = true;
      }
      else {
        final nowDate    = DateTime.now();
        final qrcodeDate = DateTime.parse(result.expirationDate);
        result.isUnExpired = qrcodeDate.isAfter(nowDate) ? true : false ;
      }

      // 用户授权判断
      bool isHaveUser = false;
      for(int i = 0 ; i < userIdList.length ; i++) {
        int uid = userIdList[i];
        if(userId == uid) {
          isHaveUser = true;
          break;
        }
      }
      result.isAuthorized = isHaveUser;
      result.authorizedState = isHaveUser == true ? '已授权' : '未授权';

      // 开通按钮可用判断
      if(result.isRecognized && result.isAuthorized && result.isUnExpired) {
        result.isCanOpen = true;
      }
      else {
        result.isCanOpen = false;
      }
      
      // 获取产品详情
      APIProduct.getProductDetailByProductId(productId,onFail: (_){
        result.isProductExist = false;
        if(onFail != null) onFail(result,'产品已下架');
      },
      onSucc: (product){
        result.isProductExist = true;
        result.productName = product.name ?? '';
        result.product = product;

        // 获取 SKU
        APIProductSKU.getProductSKUById(skuId,onFail:(_){
          if(onFail != null) onFail(result,'预置参数已被删除');
          },onSucc: (productSKU){
            result.productSKU = productSKU;
            result.isProductSkuExist = true;
            if(onSucc != null) onSucc(result,result.isAuthorized == true ? '力联码已授权' : '力联码未授权');
          },
        );

      });

    } catch (e) {
      if(onFail != null) onFail(
        LKQRCodeResult(isCanOpen: false,isForEver: false,isAuthorized: false,isRecognized: false,isUnExpired: false,
        productId: 0,skuId: 0,expirationDate: '',productName: '',
        qrcodeData: qrcodeData,isProductExist: false,isProductSkuExist:false,authorizedState: '',
      ),APITip.apiqrNoCheck);
      return;
    }
  }

  // 开通方案(含上报原始二维码报文)
  static openSchemeByScanQRcode(LKQRCodeResult qrcodeResult,String schemeName,{Function() onSucc,Function(String) onFail}) async {

    try {
      APIScheme.commitScheme(schemeName, qrcodeResult.skuId,onFail: (_){
        if(onFail != null) onFail(APITip.apiOpenFail);
        return;
      },
      onSucc: (){
        if(onSucc != null) onSucc();
        return;
      }
      );

    } catch (e) {
      if(onFail != null) onFail(APITip.apiOpenFail);
      return;
    }

  }


  // 测试用例
  static void exampleQRCode() {


    var map = Map<String,dynamic>();
    
    map['sk'] = 2;
    map['if'] = 1;
    map['ex'] = '2021-01-01';
    map['pi'] = 1;
    map['us'] = [1,2,3];

    try {
      String jsonString = json.encode(map);
      // 原始报文
      debugPrint('加密开始==>');

      debugPrint('原始明文');
      debugPrint(jsonString);
      // 加密
      String encryptWord = APIUtil.encrypted(jsonString);
      debugPrint('加密');
      debugPrint(encryptWord);
      // 加壳
      String encryptWordShell = APIUtil.shellQRcodeString(encryptWord);
      debugPrint('加壳');
      debugPrint(encryptWordShell);

      // 解密测试
      debugPrint(' ');
      debugPrint('解密开始==>');
      debugPrint('原始二维码');
      debugPrint(encryptWordShell);

      // 判断力联码
      bool isAvailable = APIUtil.isAvailableQRcode(encryptWordShell);
      debugPrint('状态判断:' + (isAvailable == true ? '是' :'不是') + '力联二维码');

      // 脱壳
      String unShellWord = APIUtil.unshellQRcodeString(encryptWordShell);
      debugPrint('脱壳');
      debugPrint(unShellWord);

      // 解密
      String plain = APIUtil.decrypted(unShellWord);
      debugPrint('解密');
      debugPrint(plain);


    } 
    catch (e) {
      
    }
}

}

// 二维码扫描结果
class LKQRCodeResult {
  
  // 可开通
  bool isCanOpen = false;
  // 已识别成功
  bool isRecognized  = false;
  // 已授权成功
  bool isAuthorized  = false;
  // 已授权文案
  String  authorizedState;
  // 有效期范围内,未超期
  bool isUnExpired;
  // 是否是永久
  bool isForEver;
  // 有效期
  String expirationDate;
  // 产品ID
  int productId;
  // skuId
  int skuId;
  // 产品存在,产品未下架
  bool isProductExist;
  // 产品Sku 是否存在
  bool isProductSkuExist;
  // 原始二维码
  String qrcodeData;
  // 产品名称
  String productName;
  // 产品返回
  Product product;
  // 产品sku
  ProductSKU productSKU;

  LKQRCodeResult({
  this.isCanOpen, this.isRecognized, this.isAuthorized, 
  this.authorizedState,this.isUnExpired, this.isForEver, 
  this.expirationDate, this.productId, this.skuId, 
  this.isProductExist,this.isProductSkuExist,
  this.qrcodeData,this.productName,this.product,this.productSKU});


}

==> leadlink_open_api-0.0.9.tar.gz: lib/api_leadlink_qrcode.dart <==
part of leadlinkopenapi;

class APILeadLinkQRCode {

  // 二维码生成字符串
  static void createQRCode(int productId,int skuId,List<int> userIdList,String expirationDate,bool isForever,{Function(String qrcodeData) onSucc,Function(String) onFail}) {

    Map<String,dynamic> input = Map<String,dynamic>();
    input['sk'] = skuId ?? 0;
    input['if'] = isForever == true ? 1 : 0;
    input['ex'] = expirationDate ?? '';
    input['pi'] = productId ?? 0;
    input['us'] = userIdList ?? [0];

    try {
      // 转字符串
      String jsonString = json.encode(input);
      // 加密
      String encryptWord = APIUtil.encrypted(jsonString);
      // 加壳
      String encryptWordShell = APIUtil.shellQRcodeString(encryptWord);
      if(onSucc != null) onSucc(encryptWordShell);
    } catch (e) {
      if(onFail != null) onFail('二维码未生成成功');
    }

  }

  // 验证是是否是力联二维码
  static bool isQRcodeAvailable(String qrcodeData) {
    return APIUtil.isAvailableQRcode(qrcodeData);
  }

  // 二维码解析
  static parseQRCode(int userId,String qrcodeData,{Function(LKQRCodeResult result,String msg) onSucc,
  Function(LKQRCodeResult result,String msg) onFail}) async {

    SharedPreferences prefs = await SharedPreferences.getInstance();
    int userId = prefs.getInt(APITip.spKeyUserId) ?? 0 ;

    // 未登录
    if(userId == 0 ) {
      if(onFail != null) onFail(
        LKQRCodeResult(isCanOpen: false,isForEver: false,isAuthorized: false,isRecognized: false,isUnExpired: false,
        productId: 0,skuId: 0,expirationDate: '',productName: '',
        qrcodeData: qrcodeData,isProductExist: false,isProductSkuExist:false,authorizedState: '',
      ),APITip.login404);
      return;
    }

    // 二维码合法性检测
    if(APIUtil.isAvailableQRcode(qrcodeData) == false) {
      if(onFail != null) onFail(
        LKQRCodeResult(isCanOpen: false,isForEver: false,isAuthorized: false,isRecognized: false,isUnExpired: false,
        productId: 0,skuId: 0,expirationDate: '',productName: '',
        qrcodeData: qrcodeData,isProductExist: false,isProductSkuExist:false,authorizedState: '',
      ),APITip.login404);
      return;
    }


    // 脱壳
    String unShellWord = APIUtil.unshellQRcodeString(qrcodeData);
    if(unShellWord == '') {
      if(onFail != null) onFail(
        LKQRCodeResult(isCanOpen: false,isForEver: false,isAuthorized: false,isRecognized: false,isUnExpired: false,
        productId: 0,skuId: 0,expirationDate: '',productName: '',
        qrcodeData: qrcodeData,isProductExist: false,isProductSkuExist:false,authorizedState: '',
      ),APITip.apiqrNoCheck);
      return;
    }
    // 解密
    String plain = APIUtil.decrypted(unShellWord);
    if(plain == '') {
      if(onFail != null) onFail(
        LKQRCodeResult(isCanOpen: false,isForEver: false,isAuthorized: false,isRecognized: false,isUnExpired: false,
        productId: 0,skuId: 0,expirationDate: '',productName: '',
        qrcodeData: qrcodeData,isProductExist: false,isProductSkuExist:false,authorizedState: '',
      ),APITip.apiqrNoCheck);
      return;
    }

    // 解析
    try {


      Map<String,dynamic> map       = json.decode(plain);
      List<int> userIdList          = map['us'].cast<int>() ?? [];
      int isForever                 = map['if'] ?? 0 ;
      int skuId                     = map['sk'] ?? 0;
      int productId                 = map['pi'] ?? 0 ;
      String expirationDate         = map['ex'] ?? '' ;

      LKQRCodeResult result = LKQRCodeResult();
      result.qrcodeData = qrcodeData;
      result.skuId      = skuId;
      result.isRecognized = true;
      result.expirationDate = expirationDate;
      result.productId = productId;
      result.isForEver = isForever == 0 ? false : true;

      // 在有效期内判断
      if(result.isForEver == true){
        result.isUnExpired = true;
      }
      else {
        final nowDate    = DateTime.now();
        final qrcodeDate = DateTime.parse(result.expirationDate);
        result.isUnExpired = qrcodeDate.isAfter(nowDate) ? true : false ;
      }

      // 用户授权判断
      bool isHaveUser = false;
      for(int i = 0 ; i < userIdList.length ; i++) {
        int uid = userIdList[i];
        if(userId == uid) {
          isHaveUser = true;
          break;
        }
      }
      result.isAuthorized = isHaveUser;
      result.authorizedState = isHaveUser == true ? '已授权' : '未授权';

      // 开通按钮可用判断
      if(result.isRecognized && result.isAuthorized && result.isUnExpired) {
        result.isCanOpen = true;
      }
      else {
        result.isCanOpen = false;
      }
      
      // 获取产品详情
      APIProduct.getProductDetailByProductId(productId,onFail: (_){
        result.isProductExist = false;
        if(onFail != null) onFail(result,'产品已下架');
      },
      onSucc: (product){
        result.isProductExist = true;
        result.productName = product.name ?? '';
        result.product = product;

        // 获取 SKU
        APIProductSKU.getProductSKUById(skuId,onFail:(_){
          if(onFail != null) onFail(result,'预置参数已被删除');
          },onSucc: (productSKU){
            result.productSKU = productSKU;
            result.isProductSkuExist = true;
            if(onSucc != null) onSucc(result,result.isAuthorized == true ? '力联码已授权' : '力联码未授权');
          },
        );

      });

    } catch (e) {
      if(onFail != null) onFail(
        LKQRCodeResult(isCanOpen: false,isForEver: false,isAuthorized: false,isRecognized: false,isUnExpired: false,
        productId: 0,skuId: 0,expirationDate: '',productName: '',
        qrcodeData: qrcodeData,isProductExist: false,isProductSkuExist:false,authorizedState: '',
      ),APITip.apiqrNoCheck);
      return;
    }
  }

  // 开通方案(含上报原始二维码报文)
  static openSchemeByScanQRcode(LKQRCodeResult qrcodeResult,String schemeName,{Function() onSucc,Function(String) onFail}) async {

    try {
      APIScheme.commitScheme(schemeName, qrcodeResult.skuId,qrcodeResult.isForEver,qrcodeResult.expirationDate,onFail: (_){
        if(onFail != null) onFail(APITip.apiOpenFail);
        return;
      },
      onSucc: (){
        if(onSucc != null) onSucc();
        return;
      }
      );

    } catch (e) {
      if(onFail != null) onFail(APITip.apiOpenFail);
      return;
    }

  }


  // 测试用例
  static void exampleQRCode() {


    var map = Map<String,dynamic>();
    
    map['sk'] = 2;
    map['if'] = 1;
    map['ex'] = '2021-01-01';
    map['pi'] = 1;
    map['us'] = [1,2,3];

    try {
      String jsonString = json.encode(map);
      // 原始报文
      debugPrint('加密开始==>');

      debugPrint('原始明文');
      debugPrint(jsonString);
      // 加密
      String encryptWord = APIUtil.encrypted(jsonString);
      debugPrint('加密');
      debugPrint(encryptWord);
      // 加壳
      String encryptWordShell = APIUtil.shellQRcodeString(encryptWord);
      debugPrint('加壳');
      debugPrint(encryptWordShell);

      // 解密测试
      debugPrint(' ');
      debugPrint('解密开始==>');
      debugPrint('原始二维码');
      debugPrint(encryptWordShell);

      // 判断力联码
      bool isAvailable = APIUtil.isAvailableQRcode(encryptWordShell);
      debugPrint('状态判断:' + (isAvailable == true ? '是' :'不是') + '力联二维码');

      // 脱壳
      String unShellWord = APIUtil.unshellQRcodeString(encryptWordShell);
      debugPrint('脱壳');
      debugPrint(unShellWord);

      // 解密
      String plain = APIUtil.decrypted(unShellWord);
      debugPrint('解密');
      debugPrint(plain);


    } 
    catch (e) {
      
    }
}

}

// 二维码扫描结果
class LKQRCodeResult {
  
  // 可开通
  bool isCanOpen = false;
  // 已识别成功
  bool isRecognized  = false;
  // 已授权成功
  bool isAuthorized  = false;
  // 已授权文案
  String  authorizedState;
  // 有效期范围内,未超期
  bool isUnExpired;
  // 是否是永久
  bool isForEver;
  // 有效期
  String expirationDate;
  // 产品ID
  int productId;
  // skuId
  int skuId;
  // 产品存在,产品未下架
  bool isProductExist;
  // 产品Sku 是否存在
  bool isProductSkuExist;
  // 原始二维码
  String qrcodeData;
  // 产品名称
  String productName;
  // 产品返回
  Product product;
  // 产品sku
  ProductSKU productSKU;

  LKQRCodeResult({
  this.isCanOpen, this.isRecognized, this.isAuthorized, 
  this.authorizedState,this.isUnExpired, this.isForEver, 
  this.expirationDate, this.productId, this.skuId, 
  this.isProductExist,this.isProductSkuExist,
  this.qrcodeData,this.productName,this.product,this.productSKU});


}
